/*******************************************************************************

  Example Plugin Library (DLL) for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    12/05/2000 20:36:48 - First Creation
    03/25/2001 17:11:55 - Conversion to v2.1
    06/20/2004 01:10:00 - DIV plugin interface v5

*******************************************************************************/

#include <windows.h>

#include "div5_filters_plugin.h"

TDIVPluginHeader PluginHeader = {
  sizeof( TDIVPluginHeader ),
  5, // plugin interface version - MUST BE 5
  DIV_PLUGIN_TYPE_FILTER,
  1, // plugin version
  5, // four filters
  "DIMIN Filter Example (VC++)"
};

//------------------------------------------------------------------------------
// About procedure
//------------------------------------------------------------------------------
void __stdcall aboutPlugin( HWND hwndParent, TDIVParameters *pars ) 
{
  MessageBox(GetFocus(), "DIMIN Filter Example (MFC)", "About Plug-in", MB_OK | MB_ICONEXCLAMATION);
}

//------------------------------------------------------------------------------
// Filter 1 shows simple no parameter filter
//------------------------------------------------------------------------------

TDIVFilterHeader FilterHeader1 = {
  sizeof(TDIVFilterHeader), // size of currently used struct
  "EXMPL_RAINT_RED",
  "24bpp Image - Paint in red",
  filter1, // pointer to a TPluginFilter function
  0,       // Bpp needed by filter, 0 - any, 888 - Gray, 111-mono
  NULL,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin     // TDivAboutProc   aboutProc    // about proc provided by filter
};


//------------------------------------------------------------------------------
// Filter 2 defines parameters to be set by DIV
//------------------------------------------------------------------------------

TDIVTagedArgument filter2ArgsArr[7] = {
  TDIVTagedArgument(
  1000,           // id number, plugin may use ids above 1000 as private
  DIM_TAG_LONG,   // TDimTagType
  1,              // number of records of type tagType pointed by tagData
  "Color value:",  // text description of the argument
  128,   // pointer to data, if data size is <= 4 bytes then it's directly stored into pointer
  0,     // pointer to min val, used if needed to define a special value retrieval, e.g. -100 .. 100
  255    // pointer to max val
  ),
  TDIVTagedArgument(
  1001,           // id number, plugin may use ids above 1000 as private
  DIM_TAG_FLOAT,  // TDimTagType
  1,              // number of records of type tagType pointed by tagData
  "Float value:",  // text description of the argument
  TDIVDataValue(5.5f),   // pointer to data, if data size is <= 4 bytes then it's directly stored into pointer
  TDIVDataValue(0.0f),     // pointer to min val, used if needed to define a special value retrieval, e.g. -100 .. 100
  TDIVDataValue(10.3f)    // pointer to max val
  ),
  TDIVTagedArgument(
  1002,           // id number, plugin may use ids above 1000 as private
  DIM_TAG_BOOL,  // TDimTagType
  1,              // number of records of type tagType pointed by tagData
  "Bool value",  // text description of the argument
  TDIVDataValue(1),   // pointer to data, if data size is <= 4 bytes then it's directly stored into pointer
  TDIVDataValue(0),   // pointer to min val, used if needed to define a special value retrieval, e.g. -100 .. 100
  TDIVDataValue(1)    // pointer to max val
  ),
  TDIVTagedArgument(
  1003,           // id number, plugin may use ids above 1000 as private
  DIM_TAG_LONG,   // TDimTagType
  1,              // number of records of type tagType pointed by tagData
  "Long value:",  // text description of the argument
  TDIVDataValue(10000),          // pointer to data, if data size is <= 4 bytes then it's directly stored into pointer
  TDIVDataValue(0),              // pointer to min val, used if needed to define a special value retrieval, e.g. -100 .. 100
  TDIVDataValue(50000)    // pointer to max val
  ),
  TDIVTagedArgument(
  1004,           // id number, plugin may use ids above 1000 as private
  DIM_TAG_LONG,   // TDimTagType
  1,              // number of records of type tagType pointed by tagData
  "Long value:",  // text description of the argument
  TDIVDataValue(545),          // pointer to data, if data size is <= 4 bytes then it's directly stored into pointer
  TDIVDataValue(0),              // pointer to min val, used if needed to define a special value retrieval, e.g. -100 .. 100
  TDIVDataValue(100000)    // pointer to max val
  ),
  TDIVTagedArgument(
  1005,           // id number, plugin may use ids above 1000 as private
  DIM_TAG_ENUM,   // TDimTagType
  1,              // number of records of type tagType pointed by tagData
  "Enum value:",  // text description of the argument
  TDIVDataValue(1),   // indicates default value
  "low|medium|high\0",            // pointer to a string with pipe separated elements: "one|two|three\0"
  TDIVDataValue(3)    // in enum case max tag indicates max number of elements
  ),
  TDIVTagedArgument(
  1006,           // id number, plugin may use ids above 1000 as private
  DIM_TAG_LONG,   // TDimTagType
  1,              // number of records of type tagType pointed by tagData
  "Long value:",  // text description of the argument
  TDIVDataValue(111),          // pointer to data, if data size is <= 4 bytes then it's directly stored into pointer
  TDIVDataValue(0),              // pointer to min val, used if needed to define a special value retrieval, e.g. -100 .. 100
  TDIVDataValue(1000)    // pointer to max val
  )
};


TDIVArguments filter2Args = {
  1,          // 0 - no preview window, 1 - DIV will open preview
  7,          // number of arguments in the array
  filter2ArgsArr, // pointer to array of argumants
  NULL, // custom controls creation INIT
  NULL, // custom controls creation QUIT
};

TDIVFilterHeader FilterHeader2 = {
  sizeof(TDIVFilterHeader),
  "EXMPL_RAINT_COLOR",
  "Gray Image - Paint in Color",
  filter2, // filter
  888,     // 8bpp needed
  &filter2Args, // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

//------------------------------------------------------------------------------
// Filter 3 shows how to create a new image using GDI
//------------------------------------------------------------------------------

TDIVFilterHeader FilterHeader3 = {
  sizeof(TDIVFilterHeader),
  "EXMPL_NEW_PAL_IMG",
  "8bpp - New paletted image",
  filter3, // filter
  0,       // 8bpp needed
  NULL,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

//------------------------------------------------------------------------------
// Filter 4 shows how to create custom controls in the window provided by DIV
//------------------------------------------------------------------------------

TDIVArguments filter4Args = {
  1,          // 0 - no preview window, 1 - DIV will open preview
  1,          // number of arguments in the array
  NULL, // don't really need anything here, we set our arguments ourselves
        // using createArgControls4
  createArgControls4, // custom controls creation INIT
  freeArgControls4, // custom controls creation QUIT
};

TDIVFilterHeader FilterHeader4 = {
  sizeof(TDIVFilterHeader),
  "EXMPL_CREATE_CNTRL",
  "Create custom controls",
  filter4, // filter
  0,     
  &filter4Args,
  aboutPlugin
};


//------------------------------------------------------------------------------
// Filter 5 rotates provided image by 90 degrees
//------------------------------------------------------------------------------

TDIVFilterHeader FilterHeader5 = {
  sizeof(TDIVFilterHeader),
  "EXMPL_ROT_90_DEG",
  "Rotate image by 90 degrees",
  filter5, // filter
  0,       // no bpp needed
  NULL,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};


//---------------------------------------------------------------------------

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
  switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


TDIVPluginHeader* __stdcall divGetPluginHeader()
{
   return &PluginHeader;
}

TDIVFilterHeader* __stdcall divGetFilterHeader(int I)
{
  switch (I)
  {
    case 0: return &FilterHeader1;
    case 1: return &FilterHeader2;
    case 2: return &FilterHeader3;
    case 3: return &FilterHeader4;
    case 4: return &FilterHeader5;
    default : return NULL;
  }
}

TDivPluginFilter __stdcall divGetFilter(int I)
{
  switch (I)
  {
    case 0: return filter1;
    case 1: return filter2;
    case 2: return filter3;
    case 3: return filter4;
    case 4: return filter5;
    default: return NULL;
  }
}

//---------------------------------------------------------------------------
