/*******************************************************************************

  Filters used by Example Plugin Library (DLL) for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    12/05/2000 20:36:48 - First Creation
    03/25/2001 17:11:55 - Conversion to v2.1
    07/07/2004 23:02:00 - Conversion v5
    
*******************************************************************************/

#include <stdio.h>
#include <windows.h>
#include <math.h>

#include <string>

#include "div5_filters_plugin.h"

#include "DIVPluginDefs.h"
#include "DIVFilterDefs.h"
#include "DIVFilterUtils.h"


//******************************************************************************
// Actually filters - Work HERE !
//******************************************************************************

//------------------------------------------------------------------------------
// Filter 1 shows simple no parameter filter
//------------------------------------------------------------------------------

int __stdcall filter1( TDIVParameters *par )
{
  int x;
  unsigned char *pline;
 
  if ( par->ver < sizeof(TDIVParameters) ) return 1;

  BITMAP bmp;

  // Retrieve the bitmap color format, width, and height.
  GetObject(par->hbmp, sizeof(BITMAP), (LPSTR)&bmp);
  if (bmp.bmBitsPixel != 24) return 0;

  pline = (unsigned char *) bmp.bmBits;
  for (x=0; x<bmp.bmHeight*bmp.bmWidthBytes; x+=3)
  {
    // next two calls only demonstrate how to call callbacks
    //showProgress( par, x, bmp.bmHeight*bmp.bmWidthBytes, "painting in Red" );
    
    pline[x+0] = 0;   // B
    //pline[x+1] = 0; // G
    pline[x+2] = 255; // R
  }

  return 0;
}

//------------------------------------------------------------------------------
// Filter 2 defines one parameter to be set by DIV
//------------------------------------------------------------------------------

int __stdcall filter2( TDIVParameters *par )
{
  if ( par->ver < sizeof(TDIVParameters) ) return 1;


  BITMAP bmp;
  // Retrieve the bitmap color format, width, and height.
  GetObject(par->hbmp, sizeof(BITMAP), (LPSTR)&bmp);
  if (bmp.bmBitsPixel != 8) return 0;

  uint colorToPaint;
  getUIntArg ( par, 0, colorToPaint );

  float fVal1;
  getFloatArg ( par, 1, fVal1 );

  bool bVal1;
  getBoolArg ( par, 2, bVal1 );

  uint uiVal1;
  getUIntArg ( par, 3, uiVal1 );

  uint uiVal2;
  getUIntArg ( par, 4, uiVal2 );

  uint uiVal3;
  getUIntArg ( par, 5, uiVal3 );

  uint uiVal4;
  getUIntArg ( par, 6, uiVal4 );


  uint x, y;
  unsigned char *pline;
  uint pB=0, pE=bmp.bmWidth-1;
  uint pI=0;

  // hbitmap is bottom-up bitmap
  pline = ( (unsigned char *) bmp.bmBits ) + (bmp.bmWidthBytes * (bmp.bmHeight-1) ); 
  for (y=0; y<(uint)bmp.bmHeight; y++)
  {
    findLineBegEndTopBot( par, bmp.bmWidth, y, pI, pB, pE);
    showProgress( par, y, bmp.bmHeight, "painting in White" );
    if ( testAbort( par ) == 1) break;
    
    for (x=pB; x<=pE; x++)
    {
      pline[x] = colorToPaint;
    }
    pline -= bmp.bmWidthBytes;    
  }

  return 0;
}


//------------------------------------------------------------------------------
// Filter 3 shows how to create a new image using GDI
//------------------------------------------------------------------------------

int __stdcall filter3( TDIVParameters *par )
{
  if ( par->ver < sizeof(TDIVParameters) ) return 1;
 

  char str[1024];
  sprintf( str, "%s", par->fileName );
  MessageBox(GetFocus(), str, "File Name", MB_OK | MB_ICONEXCLAMATION);

  // create new grey palette
  if (par->hpal != NULL) DeleteObject( par->hpal );

  // now create new bitmap
  if (par->hbmp != NULL) DeleteObject( par->hbmp );
  par->hbmp = 0;

  uint x, y;  
  unsigned int w=256, h=256;
  int bpp = 8;


  // Allocate memory for the BITMAPINFO structure. (This structure 
  // contains a BITMAPINFOHEADER structure and an array of RGBQUAD 
  // data structures.) 

  BITMAPINFO *pbmi;
  uint pbmi_size = 0;

  if (bpp != 24) 
    pbmi_size = sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * (1<< bpp); 
  else 
    // There is no RGBQUAD array for the 24-bit-per-pixel format. 
    pbmi_size = sizeof(BITMAPINFOHEADER);

  pbmi = (PBITMAPINFO) LocalAlloc(LPTR, pbmi_size ); 
  memset(pbmi, 0, pbmi_size );


  // Initialize the fields in the BITMAPINFO structure. 
  pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
  pbmi->bmiHeader.biWidth = w; 
  pbmi->bmiHeader.biHeight = h; 
  pbmi->bmiHeader.biPlanes = 1; 
  pbmi->bmiHeader.biBitCount = bpp; 
  if (bpp < 24) pbmi->bmiHeader.biClrUsed = (1<<bpp); 

  // If the bitmap is not compressed, set the BI_RGB flag. 
  pbmi->bmiHeader.biCompression = BI_RGB; 

  // Compute the number of bytes in the array of color 
  // indices and store the result in biSizeImage. 
  // For Windows NT, the width must be DWORD aligned unless 
  // the bitmap is RLE compressed. This example shows this. 
  // For Windows 95/98/Me, the width must be WORD aligned unless the 
  // bitmap is RLE compressed.
  pbmi->bmiHeader.biSizeImage = ((w * bpp +31) & ~31) /8 * h; 
  // Set biClrImportant to 0, indicating that all of the 
  // device colors are important. 
  pbmi->bmiHeader.biClrImportant = 0; 

  int i;
  for (i=0; i<256; i++)
  {
    pbmi->bmiColors[i].rgbRed   = (unsigned char) 0;
    pbmi->bmiColors[i].rgbGreen = (unsigned char) 0;
    pbmi->bmiColors[i].rgbBlue  = (unsigned char) i;
  }
  par->hpal = createHPalette( 256, pbmi->bmiColors );

   
  unsigned char *p;
  par->hbmp = CreateDIBSection(NULL, pbmi, DIB_RGB_COLORS, (void**)&p, NULL, 0);
  GdiFlush();

  
  if (p == NULL) return 1;

  unsigned char *pline = p; 
  for (y=0; y<h; y++)
  {
    for (x=0; x<w; x++)
    {
      pline[x] = y;
    }
    pline += w;    
  }

  GdiFlush();

  return 0;

}

//------------------------------------------------------------------------------
// Filter 4 shows how to create custom controls in the window provided by DIV
//------------------------------------------------------------------------------

WNDPROC lpWndProcOld; 

HWND comboBox;
int pos=0;

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  BOOL Result = FALSE;

  pos = SendMessage(comboBox, CB_GETCURSEL, 0, 0);


	if (Result != TRUE) 
    return CallWindowProc( lpWndProcOld, hwnd, message, wParam, lParam);
  else 
    return Result;
}

void __stdcall createArgControls4 ( HWND hwndParent, TDIVParameters *pars )
{
  lpWndProcOld = (WNDPROC) GetWindowLong( hwndParent, GWL_WNDPROC );
	SetWindowLong( hwndParent, GWL_WNDPROC, (long) WndProc );
  
  comboBox = CreateWindow("combobox", "combobox1", 
     WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST | WS_TABSTOP | CBS_HASSTRINGS,
     10, 10, 160, 30,
     hwndParent, NULL, (HINSTANCE)GetWindowLong(hwndParent, GWL_HINSTANCE), NULL);

  SendMessage(comboBox, CB_ADDSTRING, 0, (long)"ONE");
  SendMessage(comboBox, CB_ADDSTRING, 0, (long)"TWO");
  SendMessage(comboBox, CB_ADDSTRING, 0, (long)"THREE");
  SendMessage(comboBox, CB_SETCURSEL, 0, 0); 
}

void __stdcall freeArgControls4 ( HWND hwndParent, TDIVParameters *pars )
{
  DestroyWindow( comboBox );
}

int __stdcall filter4( TDIVParameters *par )
{
  if ( par->ver < sizeof(TDIVParameters) ) return 1;

  std::string str = "";


  switch( pos )
  {
    case 0: str = "ONE"; break;
    case 1: str = "TWO"; break;
    case 2: str = "THREE"; break;
  }
  MessageBox(GetFocus(), str.c_str(), "Index", MB_OK | MB_ICONEXCLAMATION);  

  return 0;
}


//------------------------------------------------------------------------------
// Filter 5 rotates provided image by 90 degrees
//------------------------------------------------------------------------------

int __stdcall filter5( TDIVParameters *par )
{
  if ( par->ver < sizeof(TDIVParameters) ) return 1;
  if (par->hbmp == 0) return 1;

  //----------------------------------------------
  // first obtain input image parameters
  //----------------------------------------------
  HBITMAP inhbmp = par->hbmp ;
  BITMAP inbmp;
  GetObject(par->hbmp, sizeof(BITMAP), (LPSTR)&inbmp);

  uint x, y;  
  unsigned int w=inbmp.bmHeight, h=inbmp.bmWidth;
  uint bpp = inbmp.bmBitsPixel * inbmp.bmPlanes;
  unsigned char *inbits = (unsigned char *) inbmp.bmBits;

  //unsigned int inLineSize = inbmp.bmWidthBytes;


  //----------------------------------------------
  // now allocate new image in memory
  //----------------------------------------------

  // Allocate memory for the BITMAPINFO structure. (This structure 
  // contains a BITMAPINFOHEADER structure and an array of RGBQUAD 
  // data structures.) 
  BITMAPINFO *pbmi;
  uint pbmi_size = 0;

  if (bpp != 24) 
    pbmi_size = sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * (1<< bpp); 
  else 
    // There is no RGBQUAD array for the 24-bit-per-pixel format. 
    pbmi_size = sizeof(BITMAPINFOHEADER);

  pbmi = (PBITMAPINFO) LocalAlloc(LPTR, pbmi_size ); 
  memset(pbmi, 0, pbmi_size );


  // Initialize the fields in the BITMAPINFO structure. 
  pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
  pbmi->bmiHeader.biWidth = w; 
  pbmi->bmiHeader.biHeight = h; 
  pbmi->bmiHeader.biPlanes = 1; 
  pbmi->bmiHeader.biBitCount = bpp; 

  if (bpp < 24) 
    pbmi->bmiHeader.biClrUsed = (1<<bpp);
  else
    pbmi->bmiHeader.biClrUsed = 0;

  // If the bitmap is not compressed, set the BI_RGB flag. 
  pbmi->bmiHeader.biCompression = BI_RGB; 

  // Compute the number of bytes in the array of color 
  // indices and store the result in biSizeImage. 
  // For Windows NT, the width must be DWORD aligned unless 
  // the bitmap is RLE compressed. This example shows this. 
  // For Windows 95/98/Me, the width must be WORD aligned unless the 
  // bitmap is RLE compressed.
  pbmi->bmiHeader.biSizeImage = ((w * bpp +31) & ~31) /8 * h; 
  // Set biClrImportant to 0, indicating that all of the 
  // device colors are important. 
  pbmi->bmiHeader.biClrImportant = 0; 

  //----------------------------------------------
  // create palette if needed
  //----------------------------------------------

  if (bpp != 24)
  {
    int i, pals;
    PALETTEENTRY pal[256];

    pals = GetPaletteEntries( par->hpal, 0, 256, pal );

    if (pals > 0)
      for (i=0; i<256; i++)
      {
        pbmi->bmiColors[i].rgbRed   = pal[i].peRed;
        pbmi->bmiColors[i].rgbGreen = pal[i].peGreen;
        pbmi->bmiColors[i].rgbBlue  = pal[i].peBlue;
      }
    else
      for (i=0; i<256; i++)
      {
        pbmi->bmiColors[i].rgbRed   = (unsigned char) i;
        pbmi->bmiColors[i].rgbGreen = (unsigned char) i;
        pbmi->bmiColors[i].rgbBlue  = (unsigned char) i;
      }

  } // if paletted
  
  unsigned char *p;
  par->hbmp = CreateDIBSection(NULL, pbmi, DIB_RGB_COLORS, (void**)&p, NULL, 0);
  GdiFlush();
  if (p == NULL) return 1;


  unsigned char *inP  = inbits;
  unsigned char *outP = p;
  
  // now copy image rotating it
  unsigned int inLineSize = inbmp.bmWidthBytes;
  unsigned int outLineSize = ((w * bpp +31) & ~31) /8;

  try
  {

  if (bpp == 24)
  for (y=0; y<w; y++) // we go by old image height
  {
    showProgress( par, y, w, "Left rotation" );
    if ( testAbort( par ) == 1) break;

    outP = p+(((w-1)-y)*3);
    unsigned char *ip = inP;
    for (x=0; x<h; x++) // we go by old image width
    {
      outP[0] = *ip; ip++;
      outP[1] = *ip; ip++;
      outP[2] = *ip; ip++;
      outP += outLineSize;
    }
    inP += inLineSize;
  }

  if (bpp == 16)
  for (y=0; y<w; y++) // we go by old image height
  {
    showProgress( par, y, w, "Left rotation" );
    if ( testAbort( par ) == 1) break;

    outP = p+(((w-1)-y)*2);
    unsigned char *ip = inP;
    for (x=0; x<h; x++) // we go by old image width
    {
      outP[0] = *ip; ip++;
      outP[1] = *ip; ip++;
      outP[2] = *ip; ip++;
      outP += outLineSize;
    }
    inP += inLineSize;
  }

  if (bpp == 8)
  for (y=0; y<w; y++) // we go by old image height
  {
    showProgress( par, y, w, "Left rotation" );
    if ( testAbort( par ) == 1) break;

    outP = p+((w-1)-y);
    unsigned char *ip = inP;
    for (x=0; x<h; x++) // we go by old image width
    {
      outP[0] = *ip; ip++;
      outP += outLineSize;
    }
    inP += inLineSize;
  }

  /*
  if (bpp == 4)
  {
    w = floor(w/2);
    //h = ceil(h/2);
    for (y=0; y<w; y++) // we go by old image height
    {
      showProgress( par, y, w, "Left rotation" );
      if ( testAbort( par ) == 1) break;

      outP = p+((w-1)-y);
      unsigned char *ip = inP;
      for (x=0; x<h; x++) // we go by old image width
      {
        outP[0] = *ip; ip++;
        outP += outLineSize;
      }
      inP += inLineSize;
    }
  } // 4 bpp

  if (bpp == 1)
  {
    w = ceil(w/8);
    h = ceil(h/8);
    for (y=0; y<w; y++) // we go by old image height
    {
      showProgress( par, y, w, "Left rotation" );
      if ( testAbort( par ) == 1) break;

      outP = p+((w-1)-y);
      unsigned char *ip = inP;
      for (x=0; x<h; x++) // we go by old image width
      {
        outP[0] = *ip; ip++;
        outP += outLineSize;
      }
      inP += inLineSize;
    }
  } // 1 bpp
*/

  if (inhbmp != NULL) DeleteObject( inhbmp );

  GdiFlush();

  }
  catch ( ... )
  {
    return 1;
  }

  return 0;

}





